// VIM demo engine - console class
// Copyright (c) 1999-2001 by Gergely Kutenics - gem@mailbox.hu

#include "VimCon.h"

VimCon::VimCon()
{
	Init("vim.log");
}

VimCon::VimCon(const char *log)
{
	Init(log);
	Log(1);
}

VimCon::~VimCon()
{
	Print(0);
	Log(0);
	Buffer(0);
}

void VimCon::Init(const char *log)
{
	nLogFile=NULL;
	nLogHandle=NULL;
	nPrint=0;
	nLog=0;
	nBuffer=0;
	if (log) LogFile(log);
	BufferSize(32000);
	nMessageSize=1024;
	Msg=new char[nMessageSize];
}


//
// messaging
//

void VimCon::Print(bool n)
{
	nPrint=n;
}

void VimCon::LogFile(const char *s)
{
	if (nLogFile) free(nLogFile);
	nLogFile=strdup(s);
}

void VimCon::Log(bool n)
{
	if ((n)&&(!nLog)&&(!nLogHandle)) {
		nLogHandle=fopen(nLogFile,"a");
		if (!nLogHandle) return;
		nLog=1;
		Message("\n------------------------------------------------\n\n");
	} else if ((!n)&&(nLog)&&(nLogHandle)) {
//		Message("\n\n""log closed.\n");
		fclose(nLogHandle);
		nLogHandle=NULL;
		nLog=0;
	}
}

void VimCon::Buffer(bool n)
{
	if (n) {
		if (nBuffer) Buffer(0);
		Buf=new char[nBufferSize];
		Buf[0]=0;
		nBufferPos=0;
	} else {
		if (Buf) delete Buf;
		Buf=NULL;
	}

	nBuffer=n;
}

// sets the size of the message buffer
// the contents of the buffer are lost during the reset!
void VimCon::BufferSize(long l)
{
	bool prev=nBuffer;
	Buffer(0);
	nBufferSize= (l>3000) ? l : 3000; // minimal size of the buffer
	Buffer(prev);
}

void VimCon::Message(const char *s,...)
{
	va_list argptr;
	va_start(argptr,s);
	vsprintf(Msg,s,argptr);
	va_end(argptr);

	int l=strlen(Msg);
	if (l>nMessageSize) return; // too long message

	if (nPrint) {
		printf(Msg);
	}

	if (nLog) {
		fprintf(nLogHandle,Msg);
		fflush(nLogHandle);
	} 

	if (nBuffer)  {
		// prevent buffer owerflow: if there was not enough space
		// for the message, remove the first 2/3 of the content of the buffer
		if (nBufferPos+2*l+100>=nBufferSize) {
			int ca=strlen(Buf);
			strcpy(Buf,"message buffer overflow.\r\n"
				"The earliest messages are removed.\r\n"
				"---------------------------------------------\r\n");
			int cc=nBufferSize*2/3;
			if (nBufferPos>cc) strcat(Buf,Buf+cc);
			nBufferPos=strlen(Buf);
		}

		// append string to buffer, expand line breaks from \n to \r\n
		for (int n=0;Msg[n];n++) {
			if (Msg[n]=='\n') Buf[nBufferPos++]='\r';
			Buf[nBufferPos++]=Msg[n];
		}
		Buf[nBufferPos]=0;
	}

	nMessagePos+=l;
}

void VimCon::Warning(const char *s,...)
{
	char *s2=new char[nMessageSize];
	va_list argptr;
	va_start(argptr,s);
	vsprintf(s2,s,argptr);
	va_end(argptr);
	Message(s2);
	MessageBox(NULL,s2,"Vim",MB_OK);
	delete s2;
}

void VimCon::Warning2(const char *head,const char*s,...)
{
	char *s2=new char[nMessageSize];
	va_list argptr;
	va_start(argptr,s);
	vsprintf(s2,s,argptr);
	va_end(argptr);
	Message(s2);
	MessageBox(NULL,s2,head,MB_OK);
	delete s2;
}

void VimCon::Error(const char *s,...)
{
	char *s2=new char[nMessageSize];
	va_list argptr;
	va_start(argptr,s);
	vsprintf(s2,s,argptr);
	va_end(argptr);

	Buffer(0);
	Message("\nERROR: %s\n",s2);
	MessageBox(0,s2,"Error",MB_ICONEXCLAMATION|MB_OK);
	delete s2;
	Log(0);
	Print(0);

	exit(1);
}


